package com.sun.messaging.jmq.jmsserver.data;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import com.sun.messaging.jmq.jmsserver.util.BrokerException;
import com.sun.messaging.jmq.util.JMQXid;

public class LocalTransaction extends BaseTransaction {

	public LocalTransaction() {
		super(BaseTransaction.LOCAL_TRANSACTION_TYPE);

	}
	
	public LocalTransaction(TransactionUID id, int state, JMQXid xid,
			TransactionWork txnWork) {
		super(BaseTransaction.LOCAL_TRANSACTION_TYPE);

		setTransactionWork(txnWork);
		transactionDetails.setTid(id);
		transactionDetails.setState(state);
		transactionDetails.setXid(xid);
	}

	// io methods to read and write to byte array
	public void readFromBytes(byte[] data) throws IOException, BrokerException {
		ByteArrayInputStream bais = new ByteArrayInputStream(data);
		DataInputStream dis = new DataInputStream(bais);

		readData(dis);

		dis.close();
		bais.close();
	}

	public void readData(DataInputStream dis) throws IOException,
			BrokerException {
		transactionDetails.readContent(dis);
		transactionWork.readWork(dis);
	}

	public void readObjects(ObjectInputStream ois) throws IOException,
			ClassNotFoundException {

	}

	public byte[] writeToBytes() throws IOException {
		// Log all msgs and acks for producing and consuming txn
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		DataOutputStream dos = new DataOutputStream(baos);
		writeData(dos);

		dos.close();
		baos.close();

		byte[] data = baos.toByteArray();
		return data;

	}

	public void writeData(DataOutputStream dos) throws IOException {
		transactionDetails.writeContent(dos);
		transactionWork.writeWork(dos);
	}

	public void writeObjects(ObjectOutputStream oos) throws IOException {

	}

	String getPrefix() {
		return "LocalTransaction: " + Thread.currentThread().getName() + " "
				+ this.getTid();
	}

}
